@@ -231,8 +231,8 @@ fm_subdivide <- function(mesh, n = 1) {
         lapply(
           seq_len(n),
           function(k) {
-            (segm$loc[segm$idx[, 1], ] * k / (n + 1) +
-              segm$loc[segm$idx[, 2], ] * (n - k + 1) / (n +
+            (segm$loc[segm$idx[, 1], , drop = FALSE] * k / (n + 1) +
+              segm$loc[segm$idx[, 2], , drop = FALSE] * (n - k + 1) / (n +
                 1))
           }
         )
@@ -270,9 +270,9 @@ fm_subdivide <- function(mesh, n = 1) {
     segm2
   }
 
-  p1 <- mesh$loc[mesh$graph$tv[, 1], ]
-  p2 <- mesh$loc[mesh$graph$tv[, 2], ]
-  p3 <- mesh$loc[mesh$graph$tv[, 3], ]
+  p1 <- mesh$loc[mesh$graph$tv[, 1], , drop = FALSE]
+  p2 <- mesh$loc[mesh$graph$tv[, 2], , drop = FALSE]
+  p3 <- mesh$loc[mesh$graph$tv[, 3], , drop = FALSE]
 
   tri.inner.loc <-
     do.call(
@@ -306,12 +306,27 @@ fm_subdivide <- function(mesh, n = 1) {
     ),
     is.bnd = FALSE
   )
-  # I think this code assumes fm_segm filters out duplicated points?
-  new.loc <- rbind(tri.edges$loc, tri.inner.loc)
+
+  new.loc <- unique(rbind(
+    fm_unify_coords(tri.edges$loc),
+    fm_unify_coords(tri.inner.loc)
+  ))
 
   boundary2 <- split.edges(fm_segm(mesh, boundary = TRUE), n = n)
   interior2 <- split.edges(fm_segm(mesh, boundary = FALSE), n = n)
 
+  for (k in rev(seq_len(nrow(new.loc)))) {
+    if ((NROW(boundary2$loc) > 0) &&
+      any(rowSums((new.loc[rep(k, nrow(boundary2$loc)), , drop = FALSE] -
+        boundary2$loc)^2)^0.5 < 1e-4)) {
+      new.loc <- new.loc[-k, , drop = FALSE]
+    } else if ((NROW(interior2$loc) > 0) &&
+      any(rowSums((new.loc[rep(k, nrow(interior2$loc)), , drop = FALSE] -
+        interior2$loc)^2)^0.5 < 1e-4)) {
+      new.loc <- new.loc[-k, , drop = FALSE]
+    }
+  }
+
   if (fm_manifold(mesh, "S2")) {
     radius <- mean(rowSums(mesh$loc^2)^0.5)
     renorm <- function(loc) {
